<?php

/**
 * Created by PhpStorm.
 * User: Administrator
 * Date: 2019/6/5
 * Time: 16:03
 */

namespace App\Repositories\Circle;

use App\Models\InterestCircle;
use App\Models\InterestCircleUser;
use Illuminate\Database\QueryException;
use Dingo\Api\Http\Response;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Redis;

class CircleRepository
{

    public function __construct(InterestCircle $interestCircle,
                                InterestCircleUser $interestCircleUser)
    {
        $this->interestCircle = $interestCircle;
        $this->interestCircleUser = $interestCircleUser;
    }

    /**
     * 圈子列表
     */
    public function circleLists($request, $limit = 3)
    {
        $where[] = ['is_open', 1];
        if (isset($request['is_recommend'])) {
            $where[] = ['is_recommend', 1];
        }

        return $this->interestCircle
            ->where($where)
            ->orderBy('id', 'desc')
            ->take($limit)
            ->get();
    }

    /**
     * 圈子详情
     */
    public function detail($request, $isTrashed = false)
    {
        $model = $this->interestCircle;
        if ($isTrashed) {
            $model->withTrashed();
        }
        $this->interestCircle->where('id',$request['id'])->increment('view_count');
        return $model->find($request['id']);
    }

    /**
     * 加入圈子
     * @param $request
     * @param $userInfo
     * @return array
     */
    public function joinCircle($request, $userInfo)
    {
        $row = $this->interestCircleUser
            ->where('uid', $userInfo['uid'])
            ->where('circle_id', $request['id'])
            ->first();
        if ($row) {
            return jsonError('您已经加入该圈子了');
        }
        $circle = $this->interestCircle->where('id', $request['id'])->first();
        if ($circle->join_limit) {
            $key = 'circle_error_count_' . $userInfo['uid'];
            $errorCount = Redis::get($key);
            if ($errorCount >= $circle->limit_condition) {
                return jsonError('今日次数已用完，明天再来吧');
            }
            if (empty($request['answer'])) {
                return jsonError('请输入答案~');
            }
            $question = json_decode($circle->join_question, true);
            $answer = json_decode($request['answer'], true);
            $checkRow = $this->checkQuestion($question, $answer);
            if (!$checkRow) {
                if (!$errorCount) {
                    Redis::set($key, 1);
                    $expire = Carbon::today()->endOfDay()->timestamp - Carbon::now()->timestamp;
                    Redis::expire($key, $expire);
                } else {
                    Redis::incr($key);
                }
                return jsonError('学习学习，明天再来吧~');
            }
        }
        DB::beginTransaction();
        try {
            $this->interestCircleUser->create(['uid' => $userInfo['uid'], 'circle_id' => $request['id']]);
            $this->interestCircle->where('id',$request['id'])->increment('join_count');
            DB::commit();
            //加入圈子成功后，清除错误回答次数
            $key = 'circle_error_count_' . $userInfo['uid'];
            Redis::del($key);
            return jsonSuccess();
        } catch (QueryException $exception) {
            DB::rollBack();
            return jsonError('加入圈子失败,再试试吧');
        }

    }

    /**
     * 检测答案是否正确
     * @param $question 问题
     * @param $answer   答案
     * @return bool
     */
    private function checkQuestion($question, $answer)
    {
        $result = true;
        $rightAnswer = [];
        foreach ($question as $key => $value) {
            $temp = [];
            foreach ($value['answer'] as $k => $v) {
                if ($v['right']) {
                    $temp['answer_id'][] = $k + 1;
                }
            }
            $rightAnswer[$key + 1] = $temp;
        }
//        var_dump(json_encode($rightAnswer));
        foreach ($answer as $ak => $av) {
            $right = $rightAnswer[$av['question']]['answer_id'];
//            var_dump($right);
//            var_dump($av['answer']);
//            var_dump($right != $av['answer']);
            if ($right != $av['answer']) {
                $result = false;
                break;
            }
        }
        return $result;
    }

    /**
     * 退出圈子
     * @param $request
     * @param $userInfo
     * @return array
     */
    public function exitCircle($request, $userInfo)
    {
        $info = $this->interestCircleUser
            ->where('uid', $userInfo['uid'])
            ->where('circle_id', $request['id'])
            ->first();
        if (!$info) {
            return jsonError('您未加入该圈子');
        }
        DB::beginTransaction();
        try {
            $info->delete();
            $this->interestCircle->where('id',$request['id'])->decrement('join_count');
            DB::commit();
            //退出圈子成功后，清除错误回答次数
            $key = 'circle_error_count_' . $userInfo['uid'];
            Redis::del($key);
            return jsonSuccess();
        } catch (QueryException $exception) {
            DB::rollBack();
            return jsonError('退出圈子失败,再试试吧');
        }
    }


    /**
     * 相册列表
     */
    public function pictureLists($request)
    {
        $perPage = isset($request['per_page']) ? $request['per_page'] : 20;
        $where = [];
        if (isset($request['circle_id'])) {
            $where[] = ['circle_id', $request['circle_id']];
        }
        if (isset($request['uid'])) {
            $where[] = ['uid', $request['uid']];
        }

        return $this->interestCirclePicture
            ->where($where)
            ->orderBy('created_at', 'desc')
            ->paginate($perPage);
    }

    /**
     * 删除相册图片
     */
    public function deletePicture($request)
    {
        $circle = $this->interestCirclePicture->where('id', $request['id'])->first();
        if (!$circle) {
            return Response::create([
                'message' => '获取图片信息失败',
                'status_code' => 500
            ]);
        }
        DB::beginTransaction();
        try {
            $circle->delete();
            DB::commit();
            return Response::create();

        } catch (QueryException $exception) {
            DB::rollBack();
            Log::debug('删除相册图片：' . $request['id'] . $exception->getMessage());
            return Response::create([
                'message' => '操作失败，请重试',
                'error' => $exception->getMessage(),
                'status_code' => 500
            ]);
        }
    }

}