<?php

namespace App\Service;


use Illuminate\Support\Facades\Log;
use ShaoZeMing\AliVod\SDK\CreateUploadImageRequest;
use ShaoZeMing\AliVod\SDK\GetPlayInfoRequest;
use ShaoZeMing\Aliyun\Core\DefaultAcsClient;
use ShaoZeMing\Aliyun\Core\Profile\DefaultProfile;

class AliYunVodService
{
    public function getPlayUrlByVideoId($videoId)
    {
        try {
            $client = $this->initVodClient(config('aliyunvod.vod.AccessKeyID'), config('aliyunvod.vod.AccessKeySecret'));
            $playInfo = $this->getPlayInfo($client, $videoId);
            if ($playInfo && is_object($playInfo)) {
                return $playInfo->PlayInfoList->PlayInfo[0]->PlayURL;
            } else {
                return '';
            }
        } catch (\Exception $e) {
            Log::debug('获取视频地址失败：' . $e->getMessage());
            return '';
        }
    }

    public function getImgUploadAuth()
    {
        $client = $this->initVodClient(config('aliyunvod.vod.AccessKeyID'), config('aliyunvod.vod.AccessKeySecret'));
        $imageInfo = $this->createUploadImage($client, 'default', 'jpg');
        return $imageInfo;
    }

    public function createUploadImage($client, $imageType, $imageExt)
    {
        $request = new CreateUploadImageRequest();
        $request->setImageType($imageType);
        $request->setImageExt($imageExt);
        $request->setAcceptFormat('JSON');
        return $client->getAcsResponse($request);
    }

    private function initVodClient($accessKeyId, $accessKeySecret)
    {
        $regionId = 'cn-shanghai';  // 点播服务接入区域
        $profile = DefaultProfile::getProfile($regionId, $accessKeyId, $accessKeySecret);
        return new DefaultAcsClient($profile);
    }

    private function getPlayInfo($client, $videoId)
    {
        $request = new GetPlayInfoRequest();
        $request->setVideoId($videoId);
        $request->setAcceptFormat('JSON');
        $playConfig = ['PlayDomain' => 'http://oss.uptoyo.com/'];
        $request->setPlayConfig(json_encode($playConfig));
        return $client->getAcsResponse($request);
    }
}