<?php
/**
 * Created by PhpStorm.
 * User: Administrator
 * Date: 2019/6/17
 * Time: 19:26
 */
namespace  App\Transformers\Topic;

use App\Models\Post;
use App\Models\PostCollect;
use App\Models\PostComment;
use App\Models\PostDislike;
use App\Models\PostLike;
use App\Traits\PostTrait;
use App\Traits\UserTrait;
use Carbon\Carbon;
use League\Fractal\TransformerAbstract;

class TopicPostTransformer extends TransformerAbstract
{
    use UserTrait;
    use PostTrait;
    public function __construct($uid, $invite_code)
    {
        $this->uid = $uid;
        $this->invite_code = $invite_code;
    }
    public function transform(Post $post)
    {
        $imgs = [];
        foreach($post->imgs as $img){
            $imgs[] = $img['img'];
        }
        $isLike = 0;
        $isDislike = 0;
        $isCollect = 0;
        $isFollow = 0;
        if($this->uid){
            $isLike = PostLike::where('post_id', $post['id'])->where('uid', $this->uid)->exists()?1:0;
            $isDislike = PostDislike::where('post_id', $post['id'])->where('uid', $this->uid)->exists()?1:0;
            $isCollect = PostCollect::where('post_id', $post['id'])->where('uid', $this->uid)->exists()?1:0;
            $followStatus = $this->getFollowStatus($this->uid, $post['uid']);
            if($followStatus){
                $isFollow = $followStatus;
            }
        }
        $user = $this->userInfo($post['uid']);
        return [
            'id' => $post['id'],
            'type' => $post['type'],
            'created_at' => Carbon::parse($post['created_at'])->diffForHumans(),
            'uid' => $post['uid'],
            'username' => $user['username'],
            'avatar' => $user['avatar'],
            'topic' => $this->getTopic($post['topic_ids']),
            'is_fine' => $post['is_fine'],
            'title' => $post['title'],
            'content' => $post['content'],
            'location' => $post['location'],
            'img' => $post['img'],
            'imgs' => $imgs,
            'video' => $post['video'],
            'pv' => getNumber($post->data->pv),
            'praise_count' => $post->data->praise_count,
            'comment_count' => $post->data->comment_count,
            'will_collect_bean' => $post->data->will_collect_bean + 3 * $post->data->pv,
            'is_like' => $isLike,
            'is_dislike' => $isDislike,
            'is_collect' => $isCollect,
            'comment' => $this->getNewComment($post['id'], $this->uid),
            'is_follow' => $isFollow,
            'h5url' => config('customer.share_post_h5url')."?post_id={$post['id']}&invite_code={$this->invite_code}",
            'desc_url' => $post['type'] == 'html'?config('customer.app_service_url').'/community/fragment/detail/'.$post['id']:'',
        ];
    }
}